// Description: a selector that rotates through the sequence in a fixed duration
//
//<b>notes:</b>
//<ul>
//<li> derived nodes inherit all messages from their base classes
//<li> see reset method for default settings
//<li> clients are not guarenteed to be selecting the same child at any time
//<li> this node can also be implemented with a <a href="timer.html">timer</a> node
//</ul>
//
// Category: Selection
// Author: Dave Pape
// Revision: 11/01/01
//
#include <Performer/pf/pfSwitch.h>
#include <ygNetMessages.h>
#include <ygWorld.h>
#include "localFlipbook.h"

using namespace std;

extern "C" ygNode* construct_localFlipbook(const char* name,bool master) { return new localFlipbook(name,master); }

localFlipbook::localFlipbook(const char* name,bool master) : ygNode(name,master,false)
    {
    setClassName("localFlipbook",true);
    switch_ = new pfSwitch;
    setPfNode(switch_);
    duration_ = 1.0;
    active_ = false;
    firstFrame_ = true;
    startTime_ = 0.0;
    addNetMessage("duration", &duration_, YG_NET_FLOAT);
    addNetMessage("on", &active_, YG_NET_BOOL_NEGATE);
    }

void localFlipbook::reset(void)
    {
    //reset the first frame flag
    firstFrame_ = true;
    //set duration to 1.0
    resetNetMessage("duration");
    //set active to false
    resetNetMessage("on");
    startTime_ = 0.0;
    ygNode::reset();
    }


void localFlipbook::message(const ygMessage& msg)
    {
    //set the duration of the sequence
    if (msg == "duration")
        {
        return;
        }
    //set active mode
    else if (msg == "active")
        {
        active_ = msg.boolArg(0);
        netMessageChanged("on");
        }
    else if (msg == "on")
        {
        if (active_)
            start();
        else
            stop();
        }
    else if (msg == "off")
        {
        stop();
        }
    //start the sequence
    else if (msg == "start")
        {
        start();
        }
    //stop the sequence
    else if (msg == "stop")
        {
        stop();
        }
    else
        ygNode::message(msg);
    }

void localFlipbook::start(void)
    {
    //if not active
    if (!active_)
        {
        //set active to true
        active_ = true;
        //set start time
        startTime_ = ygWorld::frameTime();
        //update the distributed active flag
        netMessageChanged("on");
        }
    }

void localFlipbook::stop(void)
    {
    //if active then set active to false
    if (active_)
        {
        active_ = false;
        //update the distributed active flag
        netMessageChanged("on");
        }
    }

void localFlipbook::app(void)
    {
    //if first frame then select first item
    if (firstFrame_)
        {
        switch_->setVal(0);
        firstFrame_ = false;
        }
    //if active
    if (active_) 
        {
        int v, max = numChildren();
        float t;
        //divide elapsed time by duration
        t = (ygWorld::frameTime() - startTime_) / duration_;
        //convert to integer
        t -= (int) t;
        v = (int) (t * max);
        if (v >= max)
            v = max - 1;
        //set selector
        switch_->setVal(v);
        }

    ygNode::app();
    }
