// Description: generates events describing node messages
//
//<b>notes:</b>
//<ul>
//<li> derived nodes inherit all messages from their base classes
//<li> see reset method for default settings
//</ul>
//
// Category: Geometry
// Author: Alex Hill
// Revision: 09/15/04 Alex Hill
//
#include <ygNodeDB.h>
#include <ygNetMessages.h>
#include "nodeMessages.h"

using namespace std;

extern "C" ygNode* construct_nodeMessages(const char* name,bool master) { return new nodeMessages(name,master); }

nodeMessages::nodeMessages(const char* name,bool master) : ygNode(name,master,false)
    {
    setClassName("nodeMessages");
    node = NULL;
    addNetMessage("node",&nodeName,YG_NET_STRING);
    }
    
void nodeMessages::reset(void)
    {
    node = NULL;
    resetNetMessage("node");
    ygNode::reset();
    }

void nodeMessages::message(const ygMessage& msg)
    {
    //generate events with message names and arguments
    if (msg == "node")
        {
        node = ygNodeDB::find(nodeName);
        if (node)
            {
            const vector<ygMessage*> messages = node->getNetMessages();
            for (int i=0;i < messages.size(); i++)
                {
                int argNum=0;
                for (int j=0; j < messages[i]->numArgTypes(); j++)
                    {
                    ygString args;
                    args = "message=";
                    args += messages[i]->message();
                    char val[32];
                    sprintf(val,"%d",i);
                    args += " number=";
                    args += val;
                    sprintf(val,"%d",j);
                    args += " argument=";
                    args += val;
                    args += " type=";
                    int type = messages[i]->typeArg(j);
                    sprintf(val,"%d",type);
                    args += val;
                    args += " data=\"";
                    if (type == YG_NET_STRING || type == YG_NET_STRING_ADD || type == YG_NET_STRING_DELETE || type == YG_NET_VECTOR)
                        {
                        args += messages[i]->stringArg(argNum++);
                        if (j == messages[i]->numArgTypes()-1)
                            {
                            int size = messages[i]->numArgs();
                            for (int k=argNum; k < size; k++)
                                {
                                args += " ";
                                args += messages[i]->stringArg(k);
                                }
                            argNum = size;
                            }
                        }
                    else if (type == YG_NET_INT || type == YG_NET_FLOAT || type == YG_NET_BOOL)
                        {
                        args += messages[i]->stringArg(argNum++);
                        }
                    else if (type == YG_NET_VEC2)
                        {
                        args += messages[i]->stringArg(argNum++);
                        args += " ";
                        args += messages[i]->stringArg(argNum++);
                        }
                    else if (type == YG_NET_VEC3)
                        {
                        args += messages[i]->stringArg(argNum++);
                        args += " ";
                        args += messages[i]->stringArg(argNum++);
                        args += " ";
                        args += messages[i]->stringArg(argNum++);
                        }
                    else if (type == YG_NET_VEC4)
                        {
                        args += messages[i]->stringArg(argNum++);
                        args += " ";
                        args += messages[i]->stringArg(argNum++);
                        args += " ";
                        args += messages[i]->stringArg(argNum++);
                        args += " ";
                        args += messages[i]->stringArg(argNum++);
                        }
                    else if (type == YG_NET_MATRIX)
                        {
                        int size = argNum + 16;
                        for (int k=argNum; k < messages[i]->numArgs(); k++)
                            {
                            args += messages[i]->stringArg(k);
                            if (k != size-1)
                                args += " ";
                            }
                        argNum += 16;
                        }
                    else if (type == YG_NET_COORD)
                        {
                        int size = argNum + 6;
                        for (int k=argNum; k < size; k++)
                            {
                            args += messages[i]->stringArg(k);
                            if (k != size-1)
                                args += " ";
                            }
                        argNum += 6;
                        }
                    args += "\"";
                    eventOccurred("changed",args);
                    }
                }
            }
        }
    else
        ygNode::message(msg);
    }

