// Description: a space that generates events in the presence of a node
//
//<b>notes:</b>
//<ul>
//<li> derived nodes inherit all messages from their base classes
//<li> see reset method for default settings
//<li> this node generates events with the name of the node detected
//</ul>
//
// Category: Trigger
// Author: Alex Hill
//           10/01/02
// Revision: 09/01/04 Alex Hill - changed to detect by documented node name instead of alias used to create the node
//
#include <set>
#include <ygNodeDB.h>
#include <ygUtil.h>
#include <ygWorld.h>
#include <ygNetMessages.h>
#include "nodeTrigger.h"

using namespace std;

extern "C" ygNode* construct_nodeTrigger(const char* name,bool master) { return new nodeTrigger(name,master); }

struct _nodeTriggerPrivateData
    {
    ygString detectType;
    set<ygNode*> prevInside;
    bool debugInside;
    int lastFrame;
    };


nodeTrigger::nodeTrigger(const char* name,bool master) : ygSpace(name,master)
    {
    setClassName("nodeTrigger");
    p_ = new struct _nodeTriggerPrivateData;
    p_->lastFrame = 0;
    debugFlag("inside", &p_->debugInside);
    addNetMessage("detect",&p_->detectType,YG_NET_STRING);
    }

nodeTrigger::~nodeTrigger(void)
    {
    delete p_;
    }


void nodeTrigger::reset(void)
    {
    //clear set of nodes previously inside
    p_->detectType.clear();
    p_->prevInside.clear();
    p_->lastFrame = ygWorld::frameNumber();
    ygSpace::reset();
    }


void nodeTrigger::message(const ygMessage& msg)
    {
    //set the type of node to detect
    if (msg == "detect")
        {
        return;
        }
    else
        ygSpace::message(msg);
    }

void nodeTrigger::setDetectType(const ygString& s)
    {
    p_->detectType = s;
    }


void nodeTrigger::app(void)
    {
    set<ygNode*> inside;
    int i;
    if (p_->lastFrame < (ygWorld::frameNumber()-1))
        p_->prevInside.clear();
    p_->lastFrame = ygWorld::frameNumber();
    const vector<ygNode*>& nodes = ygNodeDB::nodesOfType(p_->detectType);
    //for each node of detect type in the world
    for (i=0; i < nodes.size(); i++)
        {
        //if node is within the space
        if (contains(nodes[i]->origin(this)))
            {
            if (p_->debugInside)
                cout << name() << ": " << nodes[i]->name() << " is inside\n";
            //add to set of nodes inside
            inside.insert(inside.begin(),nodes[i]);
            //if inside but not previously then generate enter event
            if (p_->prevInside.find(nodes[i]) == p_->prevInside.end())
                {
                ygString args("node=");
                args += nodes[i]->name();
                //the node has entered the space
                eventOccurred("enter",args);
                }
            }
        }
    set<ygNode*>::const_iterator iter;
    //for each node previously inside
    for (iter = p_->prevInside.begin(); iter != p_->prevInside.end();
         ++iter)
        //if not currently inside then generate exit event
        if (inside.find(*iter) == inside.end())
            {
            ygString args("node=");
            args += (*iter)->name();
            //the node has exited the space
            eventOccurred("exit",args);
            }
    p_->prevInside = inside;
    ygSpace::app();
    }
