// Description: a node that creates a 3D string of text
//
//<b>notes:</b>
//<ul>
//<li> derived nodes inherit all messages from their base classes
//<li> see reset method for default settings
//<li> valid fonts are located in /usr/share/Performer/data with the extention *.mf
//</ul>
//
// Category: Geometry
// Author: Alex Hill
// Revision: 04/02/03
//
#include <Performer/pfdu.h>
#include <Performer/pr/pfGeoSet.h>
#include <Performer/pr/pfString.h>
#include <Performer/pf/pfText.h>
#include <Performer/pf/pfGroup.h>
#include <ygNetMessages.h>
#include "text.h"

using namespace std;

extern "C" ygNode* construct_text(const char* name,bool master) { return new text(name,master); }


text::text(const char* name,bool master) : ygNode(name,master)
    {
    setClassName("text",true);
    pfFont *font = pfdLoadFont_type1("/usr/share/Performer/data/Times-Elfin",PFDFONT_EXTRUDED);
    string = new pfString;
    string->setFont(font);
    text_ = new pfText;
    text_->addString(string);
    pfnode()->addChild(text_);
    netFont = "Times-Elfin";
    netColor.set(0,0,0,1);
    string->setColor(netColor[0],netColor[1],netColor[2],netColor[3]);
    netStyle = PFDFONT_EXTRUDED;
    addNetMessage("string",&netString,YG_NET_STRING);
    addNetMessage("color",&netColor,YG_NET_VEC4);
    addNetMessage("font",&netFont,YG_NET_STRING);
    addNetMessage("font",&netStyle,YG_NET_INT);
    addNetMessageMap("font","extruded",PFDFONT_EXTRUDED);
    addNetMessageMap("font","textured",PFDFONT_TEXTURED);
    addNetMessageMap("font","outlined",PFDFONT_OUTLINED);
    addNetMessageMap("font","filled",PFDFONT_FILLED);
    }

text::~text(void)
    {
    }


void text::reset(void)
    {
    resetNetMessage("string");
    string->setString(netString.c_str());
    text_->setBound(NULL,PFBOUND_DYNAMIC);
    //set default font to "Times-Elfin"
    pfFont *font = pfdLoadFont_type1("/usr/share/Performer/data/Times-Elfin",PFDFONT_EXTRUDED);
    //set default style to extruded
    resetNetMessage("font");
     string->setFont(font);
    //set color to (0,0,0,1)
    resetNetMessage("color");;
    string->setColor(netColor[0],netColor[1],netColor[2],netColor[3]);
    ygNode::reset();
    }


void text::message(const ygMessage& msg)
    {
    //set text string to be generated
    if (msg == "string")
        {
        string->setString(netString.c_str());
        text_->setBound(NULL,PFBOUND_DYNAMIC);
        }
    //set the color of the text
    else if (msg == "color")
        {
        string->setColor(netColor[0],netColor[1],netColor[2],netColor[3]);
        }
    //set the font and optional style
    else if (msg == "font")
        {
        if (netFont.length() > 0)
            {
            ygString fontPath("/usr/share/Performer/data/");
            fontPath += netFont;            
            pfFont *font = pfdLoadFont_type1(fontPath.c_str(),netStyle);
            string->setFont(font);
            }
        }
    else
        ygNode::message(msg);
    }
