// Revision: 11/01/01 Dave Pape

#include <ctype.h>
#include <Performer/pr/pfLinMath.h>
#include <QUANTA/QUANTA.hxx>
#include <map>
#include "ygNet.h"
#include "ygNetMessages.h"

using namespace std;
#define YG_NET_MAX_BUFFER_SIZE 65536

struct _ygNetMessagesData
    {
    int type;
    void* data;
    void* default_;
    };

struct _ygNetMessagesMsg
    {
    void* data;
    int datatype;
    bool reliable;
    bool broadcast;
    bool save;
    vector<struct _ygNetMessagesData*> keyData;
    map<ygString,int> flexMap;
    };

struct _ygNetMessagesPrivateData
    {
    ygString nodeName;
    QUANTAnet_datapack_c packer;
    vector<ygMessage*> messages;
    map<ygString,struct _ygNetMessagesMsg*> keys;
    };


ygNetMessages::ygNetMessages(void)
    {
    p_ = new struct _ygNetMessagesPrivateData;
    }


ygNetMessages::~ygNetMessages(void)
    {
    map<ygString,struct _ygNetMessagesMsg*>::iterator i;
    for (i=p_->keys.begin(); i != p_->keys.end(); ++i)
        delete (*i).second;
    delete p_;
    }


void ygNetMessages::setNodeName(const ygString& name)
    {
    p_->nodeName = name;
    }


void ygNetMessages::addMessage(const ygString& name,void* data,int datatype)
    {
    struct _ygNetMessagesMsg *d = p_->keys[name];
    if (!d)
        {
        d = new struct _ygNetMessagesMsg;
        p_->keys[name] = d;
        }
    struct _ygNetMessagesData *k = new struct _ygNetMessagesData;
    k->type = datatype;
    k->data = data;
    if (datatype == YG_NET_STRING || datatype == YG_NET_STRING_ADD || datatype == YG_NET_STRING_DELETE)
        {
        ygString* val = new ygString;
        *val = *((ygString*)data);
        k->default_ = val;
        }
    else if (datatype == YG_NET_INT)
        {
        int* val = (int*)malloc(sizeof(int));
        *val = *((int*)data);
        k->default_ = val;
        }
    else if (datatype == YG_NET_FLOAT)
        {
        float* val = (float*)malloc(sizeof(float));
        *val = *((float*)data);
        k->default_ = val;
        }
    else if (datatype == YG_NET_BOOL || datatype == YG_NET_BOOL_NEGATE)
        {
        bool* val = (bool*)malloc(sizeof(bool));
        *val = *((bool*)data);
        k->default_ = val;
        }
    else if (datatype == YG_NET_VEC2)
        {
        pfVec2* val = new pfVec2;
        *val = *((pfVec2*)data);
        k->default_ = val;
        }
    else if (datatype == YG_NET_VEC3)
        {
        pfVec3* val = new pfVec3;
        *val = *((pfVec3*)data);
        k->default_ = val;
        }
    else if (datatype == YG_NET_VEC4)
        {
        pfVec4* val = new pfVec4;
        *val = *((pfVec4*)data);
        k->default_ = val;
        }
    else if (datatype == YG_NET_VECTOR)
        {
        vector<float>* val = new vector<float>;
        *val = *((vector<float>*)data);
        k->default_ = val;
        }
    else if (datatype == YG_NET_MATRIX)
        {
        pfMatrix* val = new pfMatrix;
        *val = *((pfMatrix*)data);
        k->default_ = val;
        }
    else if (datatype == YG_NET_COORD)
        {
        pfCoord* val = new pfCoord;
        *val = *((pfCoord*)data);
        k->default_ = val;
        }
    d->keyData.push_back(k);
    d->reliable = true;
    if ((datatype == YG_NET_MATRIX) || (datatype == YG_NET_COORD) || (getenv("YG_NET_DEFAULT_UNRELIABLE")))
        d->reliable = false;
    d->broadcast = true;
    d->save = true;
    }

void ygNetMessages::addMessageMap(const ygString& name,const ygString& token,int result)
    {
    struct _ygNetMessagesMsg *d = p_->keys[name];
    if (!d)
        {
        cerr << "ERROR (ygNetMessages::addFlexMap): no existing flex key"
            " named '" << name << "'\n";
        return;
        }
    d->flexMap[token] = result;
    }


void ygNetMessages::dontBroadcast(const ygString& msgname)
    {
    struct _ygNetMessagesMsg *d = p_->keys[msgname];    
    if (d)
        d->broadcast = false;
    }


void ygNetMessages::dontSave(const ygString& msgname)
    {
    struct _ygNetMessagesMsg *d = p_->keys[msgname];    
    if (d)
        d->save = false;
    }


void ygNetMessages::unreliableMessage(const ygString& msgname)
    {
    struct _ygNetMessagesMsg *d = p_->keys[msgname];    
    if (d)
        d->reliable = false;
    }


void ygNetMessages::reliableMessage(const ygString& msgname)
    {
    struct _ygNetMessagesMsg *d = p_->keys[msgname];    
    if (d)
        d->reliable = true;
    }


void ygNetMessages::parseMessage(const ygMessage& msg)
    {
    struct _ygNetMessagesMsg *d = p_->keys[msg.message()];    
    if (d)
        {
    char buffer[YG_NET_MAX_BUFFER_SIZE];
        p_->packer.initPack((char*)buffer,YG_NET_MAX_BUFFER_SIZE);
        size_t size = 0;
        int argNum = 0;
        int argSize = msg.numArgs();
        int dataSize = d->keyData.size();
        for (int i=0;i<dataSize;i++)
            {
            if (d->keyData[i]->type == YG_NET_STRING)
                {
                ygString* str = (ygString*)(d->keyData[i]->data);
                if (argNum < argSize)
                    *str = msg.stringArg(argNum);
                else
                    *str = *((ygString*)(d->keyData[i]->default_));
                argNum += 1;
                if (i == dataSize-1)
                    {
                    for (int j=argNum;j<argSize;j++)
                        {
                        *str += " ";
                        *str += msg.stringArg(j);
                        }
                    }
                int len = str->length()+1;
                p_->packer.pack(str->c_str(),len);
                size += len*sizeof(char);
                }
            else if (d->keyData[i]->type == YG_NET_STRING_ADD)
                {
                if (argNum < argSize)
                    {
                    ygString* str = (ygString*)(d->keyData[i]->data);
                    ygString* token;
                    int msgSize;
                    if (i == dataSize-1)
                        msgSize = argSize;
                    else
                        msgSize = argNum+1;
                    for (int j=argNum;j<msgSize;j++)
                        {
                        int k=0;
                        int l=0;
                        bool found=false;
                        while (token = str->nextToken(" ",&l))
                            {
                            if (*token == msg.stringArg(j))
                                found = true;
                            k++;
                            delete token;
                            }
                        if (!found)
                            {
                            if (k > 0)
                                *str += " ";
                            *str += msg.stringArg(j);
                            int len = str->length()+1;
                            p_->packer.pack(str->c_str(),len);
                            size += len*sizeof(char);
                            }
                        }
                    argNum += 1;
                    }
                }
            else if (d->keyData[i]->type == YG_NET_STRING_DELETE)
                {
                if (argNum < argSize)
                    {
                    ygString* str = (ygString*)(d->keyData[i]->data);
                    ygString* token;
                    int msgSize;
                    if (i == dataSize-1)
                        msgSize = argSize;
                    else
                        msgSize = argNum+1;
                    for (int j=argNum;j<msgSize;j++)
                        {
                        ygString existing = *str;
                        str->clear();
                        int k=0;
                        int l=0;
                        bool found=false;
                        while (token = existing.nextToken(" ",&l))
                            {
                            if (*token != msg.stringArg(j))
                                {
                                if (k > 0)
                                    *str += " ";
                                *str += *token;
                                k++;
                                }
                            else
                                found = true;
                            delete token;
                            }
                        if (found)
                            {
                            int len = str->length()+1;
                            p_->packer.pack(str->c_str(),len);
                            size += len*sizeof(char);
                            }
                        }
                    argNum += 1;
                    }
                }
            else if (d->keyData[i]->type == YG_NET_INT)
                {
                int* val = (int*)(d->keyData[i]->data);
                if (argNum >= argSize)
                    *val = *((int*)(d->keyData[i]->default_));
                else if (isdigit(msg.stringArg(argNum).c_str()[0]))
                    *val = msg.intArg(argNum);
                else
                    *val = d->flexMap[msg.stringArg(argNum)];
                p_->packer.packInt(*val);
                argNum += 1;
                size += sizeof(int);
                }
            else if (d->keyData[i]->type == YG_NET_FLOAT)
                {
                float* val = (float*)(d->keyData[i]->data);
                if (argNum >= argSize)
                    *val = *((float*)(d->keyData[i]->default_));
                else
                    *val = msg.floatArg(argNum);
                p_->packer.packFloat(*val);
                argNum += 1;
                size += sizeof(float);
                }
            else if (d->keyData[i]->type == YG_NET_BOOL || d->keyData[i]->type == YG_NET_BOOL_NEGATE)
                {
                bool* val = (bool*)(d->keyData[i]->data);
                if (argNum >= argSize)
                    {
                    if (d->keyData[i]->type == YG_NET_BOOL)
                        *val = *((bool*)(d->keyData[i]->default_));
                    else
                        *val = !(*((bool*)(d->keyData[i]->default_)));
                    }
                else
                    *val = msg.boolArg(argNum);
                if (*val)
                    p_->packer.packChar('1');
                else
                    p_->packer.packChar('0');
                argNum += 1;
                size += sizeof(char);
                }
            else if (d->keyData[i]->type == YG_NET_VEC2)
                {
                pfVec2* val = (pfVec2*)(d->keyData[i]->data);
                for (int j=0;j<2;j++)
                    {
                    if (argNum >= argSize)
                        (*val)[j] = (*((pfVec2*)(d->keyData[i]->default_)))[i];
                    else
                        (*val)[j] = msg.floatArg(argNum);
                    p_->packer.packFloat((*val)[j]);
                    argNum += 1;
                    }
                size += 2*sizeof(float);
                }
            else if (d->keyData[i]->type == YG_NET_VEC3)
                {
                pfVec3* val = (pfVec3*)(d->keyData[i]->data);
                for (int j=0;j<3;j++)
                    {
                    if (argNum >= argSize)
                        (*val)[j] = (*((pfVec3*)(d->keyData[i]->default_)))[j];
                    else
                        (*val)[j] = msg.floatArg(argNum);
                    p_->packer.packFloat((*val)[j]);
                    argNum += 1;
                    }
                size += 3*sizeof(float);
                }
            else if (d->keyData[i]->type == YG_NET_VEC4)
                {
                pfVec4* val = (pfVec4*)(d->keyData[i]->data);
                for (int j=0;j<4;j++)
                    {
                    if (argNum >= argSize)
                        (*val)[j] = (*((pfVec4*)(d->keyData[i]->default_)))[j];
                    else
                        (*val)[j] = msg.floatArg(argNum);
                    p_->packer.packFloat((*val)[j]);
                    argNum += 1;
                    }
                size += 4*sizeof(float);
                }
            else if (d->keyData[i]->type == YG_NET_VECTOR)
                {
                vector<float>* val = (vector<float>*)(d->keyData[i]->data);
                int defSize = ((vector<float>*)(d->keyData[i]->default_))->size();
                int newSize = argSize-argNum;
                if (newSize < defSize)
                    newSize = defSize;
                int oldSize = ((vector<float>*)(val))->size();
                if (newSize != oldSize)
                    ((vector<float>*)(val))->resize(newSize);
                p_->packer.packInt(newSize);
                size += sizeof(int);
                for (int j=0;j<newSize;j++)
                    {
                    if (argNum >= argSize)
                        (*val)[j] = (*((vector<float>*)(d->keyData[i]->default_)))[j];
                    else
                        (*val)[j] = msg.floatArg(argNum);
                    p_->packer.packFloat((*val)[j]);
                    argNum += 1;
                    size += sizeof(float);
                    }
                }
            else if (d->keyData[i]->type == YG_NET_MATRIX)
                {
                pfMatrix * val = (pfMatrix*)(d->keyData[i]->data);
                int j,k;
                for (j=0;j<4;j++)
                    for (k=0;k<4;k++)
                        {
                        if (argNum >= argSize)
                            (*val)[j][k] = (*((pfMatrix*)(d->keyData[i]->default_)))[j][k];
                        else
                            (*val)[j][k] = msg.floatArg(argNum);
                        p_->packer.packFloat((*val)[j][k]);
                        argNum++;
                        }
                size += 16*sizeof(float);
                }
            else if (d->keyData[i]->type == YG_NET_COORD)
                {
                pfCoord * val = (pfCoord*)(d->keyData[i]->data);
                for (int j=0;j<3;j++)
                    {
                    if (argNum >= argSize)
                        val->xyz[j] = ((pfCoord*)(d->keyData[i]->default_))->xyz[j];
                    else
                        val->xyz[j] = msg.floatArg(argNum);
                    argNum += 1;
                    }
                p_->packer.packFloatArray(val->xyz.vec,3);
                for (int j=0;j<3;j++)
                    {
                    if (argNum >= argSize)
                        val->hpr[j] = ((pfCoord*)(d->keyData[i]->default_))->hpr[j];
                    else
                        val->hpr[j] = msg.floatArg(argNum);
                    argNum += 1;
                    }
                p_->packer.packFloatArray(val->hpr.vec,3);
                size += 6*sizeof(float);
                }
            else
                cerr << "ERROR (ygNetMessages::storeMessageKey): " << p_->nodeName
                    << "'s key '" << msg.message() << "' is of an unsupported type\n";
            }
        if (d->broadcast)
            ygNet::storeKey(p_->nodeName, msg.message().c_str(), buffer, size, d->reliable);
        }
    }


void ygNetMessages::resetMessage(const ygString& msgname)
    {
    struct _ygNetMessagesMsg *d = p_->keys[msgname];
    if (d)
        {
    char buffer[YG_NET_MAX_BUFFER_SIZE];
        p_->packer.initPack((char*)buffer,YG_NET_MAX_BUFFER_SIZE);
        size_t size = 0;
        bool changed=false;
        for (int i=0;i<d->keyData.size();i++)
            {
            if (d->keyData[i]->type == YG_NET_STRING || d->keyData[i]->type == YG_NET_STRING_ADD || d->keyData[i]->type == YG_NET_STRING_DELETE)
                {
                ygString* str = (ygString*)(d->keyData[i]->data);
                ygString* def = (ygString*)(d->keyData[i]->default_);
                if (*str != *def)
                    {
                    *str = *def;
                    changed = true;
                    }
                int len = str->length()+1;
                p_->packer.pack(str->c_str(),len);
                size += len*sizeof(char);
                }
            else if (d->keyData[i]->type == YG_NET_INT)
                {
                int* val = (int*)(d->keyData[i]->data);
                int* def = (int*)(d->keyData[i]->default_);
                if (*val != *def)
                    {
                    *val = *def;
                    changed = true;
                    }
                p_->packer.packInt(*val);
                size += sizeof(int);
                }
            else if (d->keyData[i]->type == YG_NET_FLOAT)
                {
                float* val = (float*)(d->keyData[i]->data);
                float* def = (float*)(d->keyData[i]->default_);
                if (*val != *def)
                    {
                    *val = *def;
                    changed = true;
                    }
                p_->packer.packFloat(*val);
                size += sizeof(float);
                }
            else if (d->keyData[i]->type == YG_NET_BOOL || d->keyData[i]->type == YG_NET_BOOL_NEGATE)
                {
                bool* val = (bool*)(d->keyData[i]->data);
                bool* def = (bool*)(d->keyData[i]->default_);
                if (*val != *def)
                    {
                    *val = *def;
                    changed = true;
                    }
                if (*val)
                    p_->packer.packChar('1');
                else
                    p_->packer.packChar('0');
                size += sizeof(char);
                }
            else if (d->keyData[i]->type == YG_NET_VEC2)
                {
                pfVec2* val = (pfVec2*)(d->keyData[i]->data);
                pfVec2* def = (pfVec2*)(d->keyData[i]->default_);
                if (*val != *def)
                    {
                    *val = *def;
                    changed = true;
                    }
                p_->packer.packFloat((*val)[0]);
                p_->packer.packFloat((*val)[1]);
                size += 2*sizeof(float);
                }
            else if (d->keyData[i]->type == YG_NET_VEC3)
                {
                pfVec3 * val = (pfVec3*)(d->keyData[i]->data);
                pfVec3* def = (pfVec3*)(d->keyData[i]->default_);
                if (*val != *def)
                    {
                    *val = *def;
                    changed = true;
                    }
                p_->packer.packFloat((*val)[0]);
                p_->packer.packFloat((*val)[1]);
                p_->packer.packFloat((*val)[2]);
                size += 3*sizeof(float);
                }
            else if (d->keyData[i]->type == YG_NET_VEC4)
                {
                pfVec4 * val = (pfVec4*)(d->keyData[i]->data);
                pfVec4* def = (pfVec4*)(d->keyData[i]->default_);
                if (*val != *def)
                    {
                    *val = *def;
                    changed = true;
                    }
                p_->packer.packFloat((*val)[0]);
                p_->packer.packFloat((*val)[1]);
                p_->packer.packFloat((*val)[2]);
                p_->packer.packFloat((*val)[3]);
                size += 4*sizeof(float);
                }
            else if (d->keyData[i]->type == YG_NET_VECTOR)
                {
                vector<float>* val = (vector<float>*)(d->keyData[i]->data);
                vector<float>* def = (vector<float>*)(d->keyData[i]->default_);
                if (*val != *def)
                    {
                    *val = *def;
                    changed = true;
                    }
                int num = ((vector<float>*)(val))->size();
                p_->packer.packInt(size);
                size += sizeof(int);
                for (int j=0;j<num;j++)
                    {
                    p_->packer.packFloat((*val)[j]);
                    size += sizeof(float);
                    }
                }
            else if (d->keyData[i]->type == YG_NET_MATRIX)
                {
                pfMatrix* val = (pfMatrix*)(d->keyData[i]->data);
                pfMatrix* def = (pfMatrix*)(d->keyData[i]->default_);
                int i,j;
                for (i=0; i<4; i++)
                    for (j=0; j<4; j++)
                        {
                        p_->packer.packFloat((*def)[i][j]);
                        if ((*val)[i][j] != (*def)[i][j])
                            {
                            changed = true;
                            (*val)[i][j] = (*def)[i][j];
                            }
                        }
                size += 16*sizeof(float);
                }
            else if (d->keyData[i]->type == YG_NET_COORD)
                {
                pfCoord* val = (pfCoord*)(d->keyData[i]->data);
                pfCoord* def = (pfCoord*)(d->keyData[i]->default_);
                if (val->xyz != def->xyz || val->hpr != def->hpr)
                    {
                    val->xyz = def->xyz;
                    val->hpr = def->hpr;
                    changed = true;
                    }
                p_->packer.packFloatArray(val->xyz.vec,3);
                p_->packer.packFloatArray(val->hpr.vec,3);
                size += 6*sizeof(float);
                }
            }
        if (changed)
            ygNet::storeKey(p_->nodeName, msgname.c_str(), buffer, size, d->reliable);
        }
    }


void ygNetMessages::sendMessage(const ygString& msgname)
    {
    struct _ygNetMessagesMsg *d = p_->keys[msgname];
    if (d)
        {
    char buffer[YG_NET_MAX_BUFFER_SIZE];
        p_->packer.initPack((char*)buffer,YG_NET_MAX_BUFFER_SIZE);
        size_t size = 0;
        for (int i=0;i<d->keyData.size();i++)
            {
            if (d->keyData[i]->type == YG_NET_STRING || d->keyData[i]->type == YG_NET_STRING_ADD || d->keyData[i]->type == YG_NET_STRING_DELETE)
                {
                ygString * str = (ygString*)(d->keyData[i]->data);
                int len = str->length()+1;
                p_->packer.pack(str->c_str(),len);
                size += len*sizeof(char);
                }
            else if (d->keyData[i]->type == YG_NET_INT)
                {
                int * val = (int*)(d->keyData[i]->data);
                p_->packer.packInt(*val);
                size += sizeof(int);
                }
            else if (d->keyData[i]->type == YG_NET_FLOAT)
                {
                float * val = (float*)(d->keyData[i]->data);
                p_->packer.packFloat(*val);
                size += sizeof(float);
                }
            else if (d->keyData[i]->type == YG_NET_BOOL || d->keyData[i]->type == YG_NET_BOOL_NEGATE)
                {
                bool * val = (bool*)(d->keyData[i]->data);
                if (*val)
                    p_->packer.packChar('1');
                else
                    p_->packer.packChar('0');
                size += sizeof(char);
                }
            else if (d->keyData[i]->type == YG_NET_VEC2)
                {
                pfVec2 * val = (pfVec2*)(d->keyData[i]->data);
                p_->packer.packFloat((*val)[0]);
                p_->packer.packFloat((*val)[1]);
                size += 2*sizeof(float);
                }
            else if (d->keyData[i]->type == YG_NET_VEC3)
                {
                pfVec3 * val = (pfVec3*)(d->keyData[i]->data);
                p_->packer.packFloat((*val)[0]);
                p_->packer.packFloat((*val)[1]);
                p_->packer.packFloat((*val)[2]);
                size += 3*sizeof(float);
                }
            else if (d->keyData[i]->type == YG_NET_VEC4)
                {
                pfVec4 * val = (pfVec4*)(d->keyData[i]->data);
                p_->packer.packFloat((*val)[0]);
                p_->packer.packFloat((*val)[1]);
                p_->packer.packFloat((*val)[2]);
                p_->packer.packFloat((*val)[3]);
                size += 4*sizeof(float);
                }
            else if (d->keyData[i]->type == YG_NET_VECTOR)
                {
                vector<float>* val = (vector<float>*)(d->keyData[i]->data);
                int num = ((vector<float>*)(val))->size();
                p_->packer.packInt(size);
                size += sizeof(int);
                for (int j=0;j<num;j++)
                    {
                    p_->packer.packFloat((*val)[j]);
                    size += sizeof(float);
                    }
                }
            else if (d->keyData[i]->type == YG_NET_MATRIX)
                {
                pfMatrix * val = (pfMatrix*)(d->keyData[i]->data);
                int i,j;
                for (i=0; i<4; i++)
                    for (j=0; j<4; j++)
                        p_->packer.packFloat((*val)[i][j]);
                size += 16*sizeof(float);
                }
            else if (d->keyData[i]->type == YG_NET_COORD)
                {
                pfCoord * val = (pfCoord*)(d->keyData[i]->data);
                p_->packer.packFloatArray(val->xyz.vec,3);
                p_->packer.packFloatArray(val->hpr.vec,3);
                size += 6*sizeof(float);
                }
            }
        ygNet::storeKey(p_->nodeName, msgname.c_str(), buffer, size, d->reliable);
        }
    }


void ygNetMessages::requestAllMessages(void)
    {
    map<ygString,struct _ygNetMessagesMsg*>::const_iterator iter;
    for (iter = p_->keys.begin(); iter != p_->keys.end(); ++iter)
        ygNet::requestKey(p_->nodeName,iter->first.c_str());
    }

bool ygNetMessages::receiveMessage(const ygString& msgname,void *newdata)
    {
    struct _ygNetMessagesMsg *d = p_->keys[msgname];
    if (d)
        {
        p_->packer.initUnpack((char*)newdata,YG_NET_MAX_BUFFER_SIZE);
        for (int i=0;i<d->keyData.size();i++)
            {
            if (d->keyData[i]->type == YG_NET_STRING || d->keyData[i]->type == YG_NET_STRING_ADD || d->keyData[i]->type == YG_NET_STRING_DELETE)
                {
                char buff[YG_NET_MAX_BUFFER_SIZE];
                int len = 0;
                p_->packer.unpackChar(&buff[len]);
                while (buff[len] != '\0')
                    {
                    len++;
                    p_->packer.unpackChar(&buff[len]);
                    }
                ygString * str = (ygString*)(d->keyData[i]->data);
                *str = buff;
                }
            else if (d->keyData[i]->type == YG_NET_INT)
                {
                int * val = (int*)(d->keyData[i]->data);
                p_->packer.unpackInt(val);
                }
            else if (d->keyData[i]->type == YG_NET_FLOAT)
                {
                float * val = (float*)(d->keyData[i]->data);
                p_->packer.unpackFloat(val);
                }
            else if (d->keyData[i]->type == YG_NET_BOOL || d->keyData[i]->type == YG_NET_BOOL_NEGATE)
                {
                bool * val = (bool*)(d->keyData[i]->data);
                char buff;
                p_->packer.unpackChar(&buff);
                if (buff == '1')
                    *val = true;
                else
                    *val = false;
                }
            else if (d->keyData[i]->type == YG_NET_VEC2)
                {
                pfVec2 * val = (pfVec2*)(d->keyData[i]->data);
                p_->packer.unpackFloat(&val->vec[0]);
                p_->packer.unpackFloat(&val->vec[1]);
                }
            else if (d->keyData[i]->type == YG_NET_VEC3)
                {
                pfVec3 * val = (pfVec3*)(d->keyData[i]->data);
                p_->packer.unpackFloat(&val->vec[0]);
                p_->packer.unpackFloat(&val->vec[1]);
                p_->packer.unpackFloat(&val->vec[2]);
                }
            else if (d->keyData[i]->type == YG_NET_VEC4)
                {
                pfVec4 * val = (pfVec4*)(d->keyData[i]->data);
                p_->packer.unpackFloat(&val->vec[0]);
                p_->packer.unpackFloat(&val->vec[1]);
                p_->packer.unpackFloat(&val->vec[2]);
                p_->packer.unpackFloat(&val->vec[3]);
                }
            else if (d->keyData[i]->type == YG_NET_VECTOR)
                {
                vector<float>* val = (vector<float>*)(d->keyData[i]->data);
                int newSize;;
                p_->packer.unpackInt(&newSize);
                int oldSize = ((vector<float>*)(val))->size();
                if (newSize != oldSize)
                    ((vector<float>*)(val))->resize(newSize);
                for (int j=0;j<newSize;j++)
                    {
                    float value;
                    p_->packer.unpackFloat(&value);
                    (*val)[j] = value;
                    }
                }
            else if (d->keyData[i]->type == YG_NET_MATRIX)
                {
                pfMatrix * val = (pfMatrix*)(d->keyData[i]->data);
                int i,j;
                for (i=0; i<4; i++)
                    for (j=0; j<4; j++)
                        p_->packer.unpackFloat(&val->mat[i][j]);
                }
            else if (d->keyData[i]->type == YG_NET_COORD)
                {
                pfCoord * val = (pfCoord*)(d->keyData[i]->data);
                p_->packer.unpackFloatArray(&val->xyz[0],3);
                p_->packer.unpackFloatArray(&val->hpr[0],3);
                }
            }
        return true;
        }
    return false;
    }    


const vector<ygMessage*>& ygNetMessages::getMessages(bool all)
    {
    for (int i=0; i<p_->messages.size(); i++)
        delete p_->messages[i];
    p_->messages.clear();
    map<ygString,struct _ygNetMessagesMsg*>::iterator j;
    for (j=p_->keys.begin(); j != p_->keys.end(); ++j)
        {
        struct _ygNetMessagesMsg *d = (*j).second;
        if (d && d->save)
            {
            ygMessage* msg = new ygMessage;
            msg->message_ = (*j).first;
            bool changed = false;
            int dataSize = d->keyData.size();
            for (int i=0;i<dataSize;i++)
                {
                if (d->keyData[i]->type == YG_NET_STRING || d->keyData[i]->type == YG_NET_STRING_ADD || d->keyData[i]->type == YG_NET_STRING_DELETE)
                    {
                    ygString *token;
                    int k=0;
                    ygString* str = (ygString*)(d->keyData[i]->data);
                    while (token = str->nextToken(" ",&k))
                        {
                        msg->args_.push_back(*token);
                        delete token;
                        }
                    ygString* def = (ygString*)(d->keyData[i]->default_);
                    if (*str != *def)
                        changed = true;
                    msg->argTypes_.push_back(d->keyData[i]->type);
                    }
                else if (d->keyData[i]->type == YG_NET_INT)
                    {
                    int* val = (int*)(d->keyData[i]->data);
                    char arg[32];
                    sprintf(arg,"%d",*val);
                    map<ygString,int>::iterator j;
                    for (j=d->flexMap.begin(); j != d->flexMap.end(); ++j)
                        {
                        if (*val == (*j).second)
                            {
                            strcpy(arg,(*j).first.c_str());
                            break;
                            }
                        }
                    msg->args_.push_back(arg);
                    int* def = (int*)(d->keyData[i]->default_);
                    if (*val != *def)
                        changed = true;
                    msg->argTypes_.push_back(YG_NET_INT);
                    }
                else if (d->keyData[i]->type == YG_NET_FLOAT)
                    {
                    float * val = (float*)(d->keyData[i]->data);
                    char arg[32];
                    sprintf(arg,"%g",*val);
                    msg->args_.push_back(arg);
                    float* def = (float*)(d->keyData[i]->default_);
                    if (*val != *def)
                        changed = true;
                    msg->argTypes_.push_back(YG_NET_FLOAT);
                    }
                else if (d->keyData[i]->type == YG_NET_BOOL)
                    {
                    bool* val = (bool*)(d->keyData[i]->data);
                    bool* def = (bool*)(d->keyData[i]->default_);
                    if (i != dataSize-1 || *val != *def)
                        {
                        if (*val)
                            msg->args_.push_back("true");
                        else
                            msg->args_.push_back("false");
                        }
                    if (*val != *def)
                        changed = true;
                    msg->argTypes_.push_back(YG_NET_BOOL);
                    }
                else if (d->keyData[i]->type == YG_NET_BOOL_NEGATE)
                    {
                    bool* val = (bool*)(d->keyData[i]->data);
                    if (dataSize != 1)
                        {
                        if (*val)
                            msg->args_.push_back("true");
                        else
                            msg->args_.push_back("false");
                        }
                    bool* def = (bool*)(d->keyData[i]->default_);
                    if (*val != *def)
                        changed = true;
                    msg->argTypes_.push_back(YG_NET_BOOL_NEGATE);
                    }
                else if (d->keyData[i]->type == YG_NET_VEC2)
                    {
                    pfVec2* val = (pfVec2*)(d->keyData[i]->data);
                    char arg[32];
                    sprintf(arg,"%g",val->vec[0]);
                    msg->args_.push_back(arg);
                    sprintf(arg,"%g",val->vec[1]);
                    msg->args_.push_back(arg);
                    pfVec2* def = (pfVec2*)(d->keyData[i]->default_);
                    if (*val != *def)
                        changed = true;
                    msg->argTypes_.push_back(YG_NET_VEC2);
                    }
                else if (d->keyData[i]->type == YG_NET_VEC3)
                    {
                    pfVec3 * val = (pfVec3*)(d->keyData[i]->data);
                    char arg[32];
                    sprintf(arg,"%g",val->vec[0]);
                    msg->args_.push_back(arg);
                    sprintf(arg,"%g",val->vec[1]);
                    msg->args_.push_back(arg);
                    sprintf(arg,"%g",val->vec[2]);
                    msg->args_.push_back(arg);
                    pfVec3* def = (pfVec3*)(d->keyData[i]->default_);
                    if (*val != *def)
                        changed = true;
                    msg->argTypes_.push_back(YG_NET_VEC3);
                    }
                else if (d->keyData[i]->type == YG_NET_VEC4)
                    {
                    pfVec4 * val = (pfVec4*)(d->keyData[i]->data);
                    char arg[32];
                    sprintf(arg,"%g",val->vec[0]);
                    msg->args_.push_back(arg);
                    sprintf(arg,"%g",val->vec[1]);
                    msg->args_.push_back(arg);
                    sprintf(arg,"%g",val->vec[2]);
                    msg->args_.push_back(arg);
                    sprintf(arg,"%g",val->vec[3]);
                    msg->args_.push_back(arg);
                    pfVec4* def = (pfVec4*)(d->keyData[i]->default_);
                    if (*val != *def)
                        changed = true;
                    msg->argTypes_.push_back(YG_NET_VEC4);
                    }
                else if (d->keyData[i]->type == YG_NET_VECTOR)
                    {
                    vector<float>* val = (vector<float>*)(d->keyData[i]->data);
                    int size = ((vector<float>*)(val))->size();
                    char arg[32];
                    for (int j=0; j<size; j++)
                        {
                        sprintf(arg,"%g",(*val)[j]);
                        msg->args_.push_back(arg);
                        }
                    vector<float>* def = (vector<float>*)(d->keyData[i]->default_);
                    if (*val != *def)
                        changed = true;
                    msg->argTypes_.push_back(YG_NET_VECTOR);
                    }
                else if (d->keyData[i]->type == YG_NET_MATRIX)
                    {
                    pfMatrix* val = (pfMatrix*)(d->keyData[i]->data);
                    pfMatrix* def = (pfMatrix*)(d->keyData[i]->default_);
                    int k,l;
                    char arg[32];
                    for (k=0; k<4; k++)
                        for (l=0; l<4; l++)
                            {
                            sprintf(arg,"%g",(*val)[k][l]);
                            msg->args_.push_back(arg);
                            if ((*val)[k][l] != (*def)[k][l])
                                changed = true;
                            }
                    msg->argTypes_.push_back(YG_NET_MATRIX);
                    }
                else if (d->keyData[i]->type == YG_NET_COORD)
                    {
                    pfCoord* val = (pfCoord*)(d->keyData[i]->data);
                    char arg[32];
                    sprintf(arg,"%g",val->xyz[0]);
                    msg->args_.push_back(arg);
                    sprintf(arg,"%g",val->xyz[1]);
                    msg->args_.push_back(arg);
                    sprintf(arg,"%g",val->xyz[2]);
                    msg->args_.push_back(arg);
                    sprintf(arg,"%g",val->hpr[0]);
                    msg->args_.push_back(arg);
                    sprintf(arg,"%g",val->hpr[1]);
                    msg->args_.push_back(arg);
                    sprintf(arg,"%g",val->hpr[2]);
                    msg->args_.push_back(arg);
                    pfCoord* def = (pfCoord*)(d->keyData[i]->default_);
                    if (val->xyz != def->xyz || val->hpr != def->hpr)
                        changed = true;
                    msg->argTypes_.push_back(YG_NET_COORD);
                    }
                }
            if (changed || all)
                p_->messages.push_back(msg);
            else
                delete msg;
            }
        }
    return p_->messages;
    }


ygString* ygNetMessages::getMessageArg(const ygString& msgname,int index)
    {
    struct _ygNetMessagesMsg *d = p_->keys[msgname];    
    if (d && d->save)
        {
        int j=0;
        for (int i=0;i<d->keyData.size();i++)
            {
            if (d->keyData[i]->type == YG_NET_STRING)
                {
                if (j == index)
                    {
                    ygString* exist = (ygString*)(d->keyData[index]->data);
                    ygString* str = new ygString;
                    *str = *exist;
                    return str;
                    }
                j++;
                }
            else if (d->keyData[i]->type == YG_NET_STRING_ADD || d->keyData[i]->type == YG_NET_STRING_DELETE)
                {
                ygString* exist = (ygString*)(d->keyData[index]->data);
                ygString* str;
                while(str = exist->nextToken(" ",&i))
                    {
                    if (j == index)
                        return str;
                    delete str;
                    j++;
                    }
                }
            else if (d->keyData[i]->type == YG_NET_INT)
                {
                if (j == index)
                    {
                    ygString* str = new ygString;
                    int* val = (int*)(d->keyData[i]->data);
                    char arg[32];
                    sprintf(arg,"%d",*val);
                    map<ygString,int>::iterator j;
                    for (j=d->flexMap.begin(); j != d->flexMap.end(); ++j)
                        {
                        if (*val == (*j).second)
                            {
                            strcpy(arg,(*j).first.c_str());
                            break;
                            }
                        }
                    *str = arg;
                    return str;
                    }
                j++;
                }
            else if (d->keyData[i]->type == YG_NET_FLOAT)
                {
                if (j == index)
                    {
                    ygString* str = new ygString;
                    float * val = (float*)(d->keyData[i]->data);
                    char arg[32];
                    sprintf(arg,"%g",*val);
                    *str = arg;
                    return str;
                    }
                j++;
                }
            else if (d->keyData[i]->type == YG_NET_BOOL || d->keyData[i]->type == YG_NET_BOOL_NEGATE)
                {
                if (j == index)
                    {
                    bool * val = (bool*)(d->keyData[i]->data);
                    ygString* str;
                    if (*val)
                        str = new ygString("true");
                    else
                        str = new ygString("false");
                    return str;
                    }
                j++;
                }
            else if (d->keyData[i]->type == YG_NET_VEC2)
                {
                pfVec2* val = (pfVec2*)(d->keyData[i]->data);
                for (int k=0; k<2; k++)
                    {
                    if (j == index)
                        {
                        ygString* str = new ygString;
                        char arg[32];
                        sprintf(arg,"%g",val->vec[k]);
                        *str = arg;
                        return str;
                        }
                    j++;
                    }
                }
            else if (d->keyData[i]->type == YG_NET_VEC3)
                {
                pfVec3* val = (pfVec3*)(d->keyData[i]->data);
                for (int k=0; k<3; k++)
                    {
                    if (j == index)
                        {
                        ygString* str = new ygString;
                        char arg[32];
                        sprintf(arg,"%g",val->vec[k]);
                        *str = arg;
                        return str;
                        }
                    j++;
                    }
                }
            else if (d->keyData[i]->type == YG_NET_VEC4)
                {
                pfVec4* val = (pfVec4*)(d->keyData[i]->data);
                for (int k=0; k<4; k++)
                    {
                    if (j == index)
                        {
                        ygString* str = new ygString;
                        char arg[32];
                        sprintf(arg,"%g",val->vec[k]);
                        *str = arg;
                        return str;
                        }
                    j++;
                    }
                }
            else if (d->keyData[i]->type == YG_NET_VECTOR)
                {
                vector<float>* val = (vector<float>*)(d->keyData[i]->data);
                int size = ((vector<float>*)(val))->size();
                for (int k=0; k<size; k++)
                    {
                    if (j == index)
                        {
                        ygString* str = new ygString;
                        char arg[32];
                        sprintf(arg,"%g",(*val)[k]);
                        *str = arg;
                        return str;
                        }
                    j++;
                    }
                }
            else if (d->keyData[i]->type == YG_NET_MATRIX)
                {
                pfMatrix * val = (pfMatrix*)(d->keyData[i]->data);
                int k,l;
                for (k=0; k<4; k++)
                    for (l=0; l<4; l++)
                        {
                        if (j == index)
                            {
                            ygString* str = new ygString;
                            char arg[32];
                            sprintf(arg,"%g",(*val)[k][l]);
                            *str = arg;
                            return str;
                            }
                        j++;
                        }
                }
            else if (d->keyData[i]->type == YG_NET_COORD)
                {
                pfCoord * val = (pfCoord*)(d->keyData[i]->data);
                for (int k=0; k<3; k++)
                    {
                    if (j == index)
                        {
                        ygString* str = new ygString;
                        char arg[32];
                        sprintf(arg,"%g",val->xyz[k]);
                        *str = arg;
                        return str;
                        }
                    j++;
                    }
                for (int k=0; k<3; k++)
                    {
                    if (j == index)
                        {
                        ygString* str = new ygString;
                        char arg[32];
                        sprintf(arg,"%g",val->hpr[k]);
                        *str = arg;
                        return str;
                        }
                    j++;
                    }
                }
            }
        }
    return NULL;
    }
